require 'sc_analytics/version'
require 'sc_analytics/query'
require 'sc_analytics/connections'
require 'sc_analytics/exceptions'

require 'date'
require 'fileutils'
require 'csv'
require 'win32ole'
require 'thread'
Thread.abort_on_exception = true

begin
  # for development environment
  require 'pry'
rescue LoadError
end

module SCAnalytics
	def run_query(*queries)
		queries.each{|qry| qry.run }
  end
  alias :run_queries :run_query
  alias :run_in_sequence :run_query

  def run_in_parallel(*queries)
    queries.each { |query| raise "cannot run forked query #{qry.name} in parallel with other queries" if query.forked? && queries.length > 1 }

    threads = []
    queries.each do |query|
      threads << Thread.new(query) do |qry|
        qry.run
      end
    end

    threads.each{|thread| thread.join }
  end

  def alert(status)
  	curr_time = Time.now
    puts "#{curr_time.strftime("%m/%d/%Y %H:%M:%S")} #{status}"
    puts
    return curr_time
  end

  def open_excel_file(file_name)
    excel_instance.Workbooks.open("#{Dir.pwd}/#{file_name}")
    nil
  end

  def run_excel_macro(macro_name, arg = nil)
    if arg
      excel_instance.run(macro_name, arg)
    else
      excel_instance.run(macro_name)
    end

    nil
  end

  def quit_excel
    excel_instance.quit
    nil
  end

  private

  def excel_instance
    @excel = WIN32OLE.new('Excel.Application') unless @excel
    @excel.visible = true
    
    @excel
  end
end