module SCAnalytics
	module Connections
		class Oracle < Connection
			attr_reader :connect_string, :service_name, :sid

			require 'oci8'
			OCI8::BindType::Mapping[:number_unknown_prec] = OCI8::BindType::BasicNumberType
			OCI8::BindType::Mapping[:number_no_prec_setting] = OCI8::BindType::BasicNumberType

			def initialize(connection_name, config)
				super

				@host = config[:connection][:description][:address][:host]
				@port = config[:connection][:description][:address][:port]
				@service_name = config[:connection][:description][:connect_data][:service_name]
				@sid = config[:connection][:description][:connect_data][:sid]

				@connect_string = generate_connect_string_for(config[:connection])
				connect
			end

			def define_col_types_for(query, cursor)
				(query.columns_to_cast || {}).each_pair do |type, cols|
					cols.each do |col|
						cursor.define(col, type)
					end
				end
			end

			def get_cursor_for(query)
				cursor = @client.parse(query.sql)
			end

			def run(query, cursor)
				cursor.exec
				define_col_types_for query, cursor

				headers = cursor.get_col_names
				results = []
				
				while row = cursor.fetch
					results << row
				end

				Struct::QueryResult.new(headers, results)
			end

			def connect
				super do
					@client = OCI8.new @username, @password, @connect_string
				end
			end

			def generate_connect_string_for(conn_hsh, conn_str = "")
				conn_hsh.each_pair do |descriptor,detail|
					conn_str << "("
					conn_str << "#{descriptor.to_s}="
					detail.is_a?(Hash) ? conn_str = generate_connect_string_for(detail, conn_str) : conn_str << detail.to_s
					conn_str << ")"
				end
				conn_str
			end

		end
	end
end